@@ -51,7 +51,6 @@ from ._node_walkers import AddressMaps, OwnedbyAddressMap
 
 from .__about__ import __version__
 
-
 file_path = os.path.dirname(__file__)
 
 
@@ -300,7 +299,17 @@ class PythonExporter:
                            asyncoutput: bool,
                            legacy_block_access: bool,
                            udp_to_include: Optional[list[str]],
-                           hide_node_func: HideNodeCallback) -> None:
+                           hide_node_func: HideNodeCallback,
+                           legacy_enum_type: bool) -> None:
+
+        def visible_nonsignal_node(node: Node) -> int:
+            count = 0
+            for child_node in node.children(unroll=False):
+                if not isinstance(child_node, SignalNode):
+                    if not hide_node_func(child_node):
+                        count +=1
+            return count
+
 
         context = {
             'print': print,
@@ -350,7 +359,9 @@ class PythonExporter:
             'dependent_property_enum':
                 self._get_dependent_property_enum(node=top_block,
                                                   udp_to_include=udp_to_include),
-            'hide_node_func': hide_node_func
+            'hide_node_func': hide_node_func,
+            'visible_nonsignal_node' : visible_nonsignal_node,
+            'legacy_enum_type': legacy_enum_type
         }
         if legacy_block_access is True:
             context['get_array_typecode'] = get_array_typecode
@@ -412,12 +423,14 @@ class PythonExporter:
                                      target_name='example.py',
                                      template_context=context)
 
+    # pylint: disable-next=too-many-arguments
     def __export_base_tests(self, *,
                             top_block: AddrmapNode,
                             package: _Package,
                             skip_lib_copy: bool,
                             asyncoutput: bool,
-                            legacy_block_access: bool) -> None:
+                            legacy_block_access: bool,
+                            legacy_enum_type: bool) -> None:
         """
 
         Args:
@@ -435,6 +448,7 @@ class PythonExporter:
             'skip_lib_copy': skip_lib_copy,
             'version': __version__,
             'legacy_block_access': legacy_block_access,
+            'legacy_enum_type': legacy_enum_type
         }
 
         context.update(self.user_template_context)
@@ -457,7 +471,9 @@ class PythonExporter:
                        asyncoutput: bool,
                        legacy_block_access: bool,
                        udp_to_include: Optional[list[str]],
-                       hide_node_func: HideNodeCallback) -> None:
+                       hide_node_func: HideNodeCallback,
+                       legacy_enum_type: bool
+                       ) -> None:
         """
 
         Args:
@@ -540,7 +556,8 @@ class PythonExporter:
                 'dependent_property_enum':
                     self._get_dependent_property_enum(node=top_block,
                                                       udp_to_include=udp_to_include),
-                'hide_node_func': hide_node_func
+                'hide_node_func': hide_node_func,
+                'legacy_enum_type': legacy_enum_type
             }
 
             self.__stream_jinja_template(template_name="addrmap_tb.py.jinja",
@@ -570,7 +587,7 @@ class PythonExporter:
                     raise RuntimeError('It is not permitted to expose a property name used to'
                                        ' build the peakrdl-python wrappers: ' + reserved_name)
 
-    # pylint: disable-next=too-many-arguments
+    # pylint: disable-next=too-many-arguments,too-many-locals
     def export(self, node: Union[RootNode, AddrmapNode], path: str, *,
                asyncoutput: bool = False,
                skip_test_case_generation: bool = False,
@@ -579,7 +596,8 @@ class PythonExporter:
                legacy_block_access: bool = True,
                show_hidden: bool = False,
                user_defined_properties_to_include: Optional[list[str]] = None,
-               hidden_inst_name_regex: Optional[str] = None) -> str:
+               hidden_inst_name_regex: Optional[str] = None,
+               legacy_enum_type: bool = True) -> str:
         """
         Generated Python Code and Testbench
 
@@ -594,8 +612,10 @@ class PythonExporter:
                                                     operations
             skip_library_copy (bool): skip copy the libraries to the generated package, this is
                                       useful to turn off when developing peakrdl python to avoid
-                                      editing the wrong copy of the library. However, it is not
-                                      recommended in end user cases
+                                      editing the wrong copy of the library. It also avoids the
+                                      GPL code being part of the package for distribution,
+                                      However, this means the end-user is responsible for
+                                      installing the libraries.
             legacy_block_access (bool): version 0.8 changed the block access methods from using
                                         arrays to to lists. This allows memory widths of other
                                         than 8, 16, 32, 64 to be supported which are legal in
@@ -613,6 +633,9 @@ class PythonExporter:
             hidden_inst_name_regex (str) : A regular expression which will hide any fully
                                            qualified instance name that matches, set to None to
                                            for this to have no effect
+            legacy_enum_type (bool): version 1.2 introduced a new Enum type that allows system
+                                     rdl ``name`` and ``desc`` properties on field encoding
+                                     to be included. The legacy mode uses python IntEnum.
 
 
         Returns:
@@ -668,7 +691,8 @@ class PythonExporter:
                                 skip_lib_copy=skip_library_copy,
                                 legacy_block_access=legacy_block_access,
                                 udp_to_include=user_defined_properties_to_include,
-                                hide_node_func=hide_node_func)
+                                hide_node_func=hide_node_func,
+                                legacy_enum_type=legacy_enum_type)
 
         self.__export_simulator(top_block=top_block, package=package, asyncoutput=asyncoutput,
                                 skip_lib_copy=skip_library_copy,
@@ -682,13 +706,15 @@ class PythonExporter:
             # export the baseclasses for the tests
             self.__export_base_tests(top_block=top_block, package=package, asyncoutput=asyncoutput,
                                      skip_lib_copy=skip_library_copy,
-                                     legacy_block_access=legacy_block_access)
+                                     legacy_block_access=legacy_block_access,
+                                     legacy_enum_type=legacy_enum_type)
             # export the tests themselves, these are broken down to one file per addressmap
             self.__export_tests(top_block=top_block, package=package, asyncoutput=asyncoutput,
                                 skip_lib_copy=skip_library_copy,
                                 legacy_block_access=legacy_block_access,
                                 udp_to_include=user_defined_properties_to_include,
-                                hide_node_func=hide_node_func)
+                                hide_node_func=hide_node_func,
+                                legacy_enum_type=legacy_enum_type)
 
         return top_block.inst_name
 
@@ -735,8 +761,18 @@ class PythonExporter:
                 # de-duplicate the values
                 raise RuntimeError("node is already in the lookup dictionary")
 
-            cand_type_name = get_fully_qualified_type_name(child_node)
+            if child_node == node:
+                # in the case of the top node the type name should match the instance name
+                cand_type_name = child_node.inst_name
+            else:
+                cand_type_name = get_fully_qualified_type_name(child_node)
             if cand_type_name in self.node_type_name.values():
+                if child_node == node:
+                    raise RuntimeError(
+                        f'Top Node name {cand_type_name} already used by instance ' \
+                        + str(list(self.node_type_name.keys())
+                              [list(self.node_type_name.values()).index(cand_type_name)])
+                    )
                 self.node_type_name[child_inst] = cand_type_name + '_0x' + hex(hash(child_inst))
             else:
                 self.node_type_name[child_inst] = cand_type_name
